@@ -1,24 +1,26 @@
 from enum import Enum
 from pathlib import Path
+from typing import Sequence
 
 from construct import Container, ListContainer
-from mercury_engine_data_structures.formats.bmmap import Bmmap
+
 from open_dread_rando.patcher_editor import PatcherEditor
 
-# copied from existing entity so we don't have to make a whole shield
+# copied from existing entity, so we don't have to make a whole shield
 _EXAMPLE_SHIELD = {"scenario": "s010_cave", "layer": "default", "actor": "Door003_missileShield"}
 
-"""
-Enum representing minimap data. Also contains functions that return mapDoor and mapBlockage Containers. 
 
-iconId: a nondirectional icon id. 'L' or 'R' is appended for each direction. 
-tuple: the offsets for the oBoxes. All values should be taken from a "left" entity:
-    0: x-axis offset from vPos to the bottom-right corner
-    1: x-axis offset from vPos to the top-left corner
-    2: y-axis offset from vPos to the bottom-right corner
-    3: y-axis offset from vPos to the top-left corner
-"""
 class MinimapIconData(Enum):
+    """
+    Enum representing minimap data. Also contains functions that return mapDoor and mapBlockage Containers.
+
+    iconId: a non-directional icon id. 'L' or 'R' is appended for each direction.
+    tuple: the offsets for the oBoxes. All values should be taken from a "left" entity:
+        0: x-axis offset from vPos to the bottom-right corner
+        1: x-axis offset from vPos to the top-left corner
+        2: y-axis offset from vPos to the bottom-right corner
+        3: y-axis offset from vPos to the top-left corner
+    """
     DOOR_FRAME = ("DoorFrame", (-150, -50, 0, 300))
     DOOR_POWER = ("DoorPower", (-150, -50, 0, 300))
     DOOR_CHARGE = ("DoorCharge", (-150, -50, 0, 300))
@@ -30,55 +32,57 @@ class MinimapIconData(Enum):
     SHIELD_MISSILE = ("BlockageMissile", (-300, -150, 0, 300))
     SHIELD_SUPER_MISSILE = ("BlockageSuperMissile", (-300, -150, 0, 300))
 
-    def __init__(self, icon_id: str, oBoxOffsets: tuple[float, float, float, float]):
+    def __init__(self, icon_id: str, offsets: tuple[float, float, float, float]):
         """
         Initializes a new MinimapIconData with the given icon id and offsets for the left-facing direction
         """
         self.icon_id = icon_id
-        self.oBox_min = (oBoxOffsets[0], oBoxOffsets[2])
-        self.oBox_max = (oBoxOffsets[1], oBoxOffsets[3])
-    
-    def create_mapDoor(self, vPos: list[float, float, float]) -> Container:
+        self.oBox_min = (offsets[0], offsets[2])
+        self.oBox_max = (offsets[1], offsets[3])
+
+    def create_map_door(self, pos: Sequence[float]) -> Container:
         """creates a mapDoor for a given element"""
         cont = Container()
-        cont["vPos"] = ListContainer([vPos[0], vPos[1]])
+        cont["vPos"] = ListContainer([pos[0], pos[1]])
         cont["oBoxL"] = Container(
-            Min=ListContainer([vPos[0] + self.oBox_min[0], vPos[1] - self.oBox_min[1]]),
-            Max=ListContainer([vPos[0] + self.oBox_max[0], vPos[1] + self.oBox_max[1]]))
+            Min=ListContainer([pos[0] + self.oBox_min[0], pos[1] - self.oBox_min[1]]),
+            Max=ListContainer([pos[0] + self.oBox_max[0], pos[1] + self.oBox_max[1]]))
         cont["oBoxR"] = Container(
-            Min = ListContainer([vPos[0] - self.oBox_max[0], vPos[1] - self.oBox_min[1]]),
-            Max = ListContainer([vPos[0] - self.oBox_min[0], vPos[1] + self.oBox_max[1]]))
+            Min=ListContainer([pos[0] - self.oBox_max[0], pos[1] - self.oBox_min[1]]),
+            Max=ListContainer([pos[0] - self.oBox_min[0], pos[1] + self.oBox_max[1]]))
         cont["sLeftIconId"] = f"{self.icon_id}L"
         cont["sRightIconId"] = f"{self.icon_id}R"
         cont["aRoomIds"] = ListContainer()
-        
+
         return cont
 
-    def create_mapBlockage(self, vPos: list[float, float, float], dir: str) -> Container:
+    def create_map_blockage(self, pos: Sequence[float], dir: str) -> Container:
         """Creates a mapBlockage for a shield in a given direction"""
         cont = Container()
-        cont["vPos"] = ListContainer([vPos[0], vPos[1]])
+        cont["vPos"] = ListContainer([pos[0], pos[1]])
 
         # get delta, which depends on shield dir. 
-        # shield L needs no adjustments so delta is zero. shield R needs to be moved oBoxMin[0] + oBoxMax[0] to the right. 
+        # shield L needs no adjustments so delta is zero.
+        # shield R needs to be moved oBoxMin[0] + oBoxMax[0] to the right.
         delta = 0 if dir == "L" else -(self.oBox_min[0] + self.oBox_max[0])
 
         cont["oBox"] = Container(
-            Min=ListContainer([vPos[0] + self.oBox_min[0] + delta, vPos[1] + self.oBox_min[1]]),
-            Max=ListContainer([vPos[0] + self.oBox_max[0] + delta, vPos[1] + self.oBox_max[1]]))
+            Min=ListContainer([pos[0] + self.oBox_min[0] + delta, pos[1] + self.oBox_min[1]]),
+            Max=ListContainer([pos[0] + self.oBox_max[0] + delta, pos[1] + self.oBox_max[1]]))
         cont["sIconId"] = f"{self.icon_id}{dir}"
         cont["bFlipX"] = False
         cont["bFlipY"] = False
 
         return cont
 
-"""
-Enum containing data on actors
 
-actordef: the actordef for the actor
-minimapData: the MinimapIconData for this type
-"""
 class ActorData(Enum):
+    """
+    Enum containing data on actors
+
+    actordef: the actordef for the actor
+    minimapData: the MinimapIconData for this type
+    """
     DOOR_FRAME = (["doorframe"], MinimapIconData.DOOR_FRAME)
     DOOR_POWER = (["doorpowerpower", "doorpowerclosed", "doorclosedpower"], MinimapIconData.DOOR_POWER)
     DOOR_CHARGE = (["doorchargecharge", "doorchargeclosed", "doorclosedcharge"], MinimapIconData.DOOR_CHARGE)
@@ -95,15 +99,16 @@ class ActorData(Enum):
         self.actordefs = [f"actors/props/{v}/charclasses/{v}.bmsad" for v in actordef]
         self.minimapData = minimap
 
-"""
-Enum containing info on each door type
 
-type: the name Randovania calls the door
-door: the door's ActorData
-need_shield: whether the actor needs a shield
-shield: the shield's ActorData
-"""
 class DoorType(Enum):
+    """
+    Enum containing info on each door type
+
+    type: the name Randovania calls the door
+    door: the door's ActorData
+    need_shield: whether the actor needs a shield
+    shield: the shield's ActorData
+    """
     FRAME = ("frame", ActorData.DOOR_FRAME)
     POWER = ("power_beam", ActorData.DOOR_POWER)
     CHARGE = ("charge_beam", ActorData.DOOR_CHARGE)
@@ -112,30 +117,38 @@ class DoorType(Enum):
     WAVE_BEAM = ("wave_beam", ActorData.DOOR_POWER, True, ActorData.SHIELD_WAVE_BEAM)
     MISSILE = ("missile", ActorData.DOOR_POWER, True, ActorData.SHIELD_MISSILE)
     SUPER_MISSILE = ("super_missile", ActorData.DOOR_POWER, True, ActorData.SHIELD_SUPER_MISSILE)
-    GRAPPLE = ("grapple_beam", ActorData.DOOR_GRAPPLE)
-    PRESENCE = ("phantom_cloak", ActorData.DOOR_PRESENCE, False, None, True, False)
+    GRAPPLE = ("grapple_beam", ActorData.DOOR_GRAPPLE, False, None, True, True, ["actors/props/door"])
+    PRESENCE = ("phantom_cloak", ActorData.DOOR_PRESENCE, False, None, True, False, ["actors/props/door"])
 
-    def __init__(self, rdv_door_type: str, shield_data: ActorData, need_shield: bool =False, shield_actordef: ActorData =None, can_be_removed: bool =True, can_be_added: bool =True):
+    def __init__(self, rdv_door_type: str, door_data: ActorData, need_shield: bool = False,
+                 shield_data: ActorData = None, can_be_removed: bool = True, can_be_added: bool = True, additional_asset_folders: list[str] = None):
         self.type = rdv_door_type
         self.need_shield = need_shield
-        self.door = shield_data
-        self.shield = shield_actordef
+        self.door = door_data
+        self.shield = shield_data
         self.can_be_removed = can_be_removed
         self.can_be_added = can_be_added
-    
+        self.required_asset_folders = [] if additional_asset_folders is None else additional_asset_folders
+        self.required_asset_folders.append(Path(self.door.actordefs[0]).parent.parent.as_posix())
+        if self.need_shield:
+            self.required_asset_folders.append(Path(self.shield.actordefs[0]).parent.parent.as_posix())
+
     @classmethod
     def get_type(cls, type: str):
-        for e in DoorType:
+        for e in cls:
             if e.type == type:
                 return e
-        
+
         raise ValueError(f"{type} is not a patchable door! Please check DoorType enum for list of patchable doors.")
 
+
 def is_door(actor: Container):
     return "LIFE" in actor.pComponents and "CDoorLifeComponent" == actor.pComponents.LIFE["@type"]
 
+
 class DoorPatcher:
     """An API to patch doors. Call patch_door() to use. """
+
     def __init__(self, editor: PatcherEditor) -> None:
         # get actors from reference dicts
         self.editor = editor
@@ -150,7 +163,7 @@ class DoorPatcher:
         # make sure it's a door
         if not is_door(door):
             raise ValueError(f"Actor {door.sName} in scenario {scenario} is not a door!")
-        
+
         # filter enums by door's actorref
         door_actor_ref = door.oActorDefLink.split(':')[1]
         possible_enum_values = list()
@@ -158,19 +171,20 @@ class DoorPatcher:
             for tdef in type.door.actordefs:
                 if door_actor_ref == tdef:
                     possible_enum_values.append(type)
-        
+
         if len(possible_enum_values) == 1:
             return possible_enum_values[0]
-        
+
         # check for shields
         shield_actor = None
         life_comp = door.pComponents.LIFE
         for link_name in ["wpLeftDoorShieldEntity", "wpRightDoorShieldEntity"]:
             if life_comp[link_name] != '{EMPTY}':
-                shield_actor = self.editor.resolve_actor_reference(self.editor.reference_for_link(life_comp[link_name], scenario))
-        
+                shield_actor = self.editor.resolve_actor_reference(
+                    self.editor.reference_for_link(life_comp[link_name], scenario))
+
         if shield_actor is not None:
-            # remove non shielded doors if shield exists
+            # remove non-shielded doors if shield exists
             possible_enum_values = [e for e in possible_enum_values if e.need_shield]
 
             # check shields
@@ -185,7 +199,6 @@ class DoorPatcher:
         else:
             raise ValueError(f"Door {door.sName} in scenario {scenario} is not a patchable door!")
 
-
     def patch_door(self, door_ref: dict, door_type: str):
         """
         Patches a door given a reference. 
@@ -194,7 +207,7 @@ class DoorPatcher:
         """
 
         scenario = door_ref["scenario"]
-        doorActor = self.editor.resolve_actor_reference(door_ref)
+        door_actor = self.editor.resolve_actor_reference(door_ref)
 
         # get the type of door we are patching to
         door_type = DoorType.get_type(door_type)
@@ -202,12 +215,13 @@ class DoorPatcher:
             raise ValueError(f"Door type {door_type} cannot be patched in!")
 
         # get the type of door we are patching
-        door_in_scenario_type = self.door_actor_to_type(doorActor, scenario)
+        door_in_scenario_type = self.door_actor_to_type(door_actor, scenario)
         if door_in_scenario_type.can_be_removed is False:
-            raise ValueError(f"Base game door {door_in_scenario_type.type} cannot be patched!\nRequested door: {door_ref['actor']} in {scenario}")
-        
-        self.door_to_basic(doorActor, door_in_scenario_type, scenario)
-        self.power_to_door_type(doorActor, door_type, scenario)
+            raise ValueError(
+                f"Base game door {door_in_scenario_type.type} cannot be patched!\nRequested door: {door_ref['actor']} in {scenario}")
+
+        self.door_to_basic(door_actor, door_in_scenario_type, scenario)
+        self.power_to_door_type(door_actor, door_type, scenario)
 
     def door_to_basic(self, door: Container, door_type: DoorType, scenario: str):
         """
@@ -215,7 +229,7 @@ class DoorPatcher:
         """
         if door_type is DoorType.POWER:
             return
-        
+
         if door_type.need_shield:
             self.remove_shields(door, scenario)
 
@@ -223,7 +237,6 @@ class DoorPatcher:
         if door_type.door is not ActorData.DOOR_POWER:
             self.any_door_to_power(door, scenario)
 
-
     # removes any shields if the door has them
     def remove_shields(self, door: Container, scenario: str):
         """
@@ -235,7 +248,8 @@ class DoorPatcher:
         life_comp = door.pComponents.LIFE
         for link_name in ["wpLeftDoorShieldEntity", "wpRightDoorShieldEntity"]:
             link = life_comp[link_name]
-            if link == '{EMPTY}': continue # skip shield if not linked
+            if link == '{EMPTY}':
+                continue  # skip shield if not linked
             self.editor.remove_entity(self.editor.reference_for_link(link, scenario), "mapBlockages")
             life_comp[link_name] = '{EMPTY}'
 
@@ -243,33 +257,32 @@ class DoorPatcher:
     def any_door_to_power(self, door: Container, scenario: str):
         door.oActorDefLink = f"actordef:{ActorData.DOOR_POWER.actordefs[0]}"
         self.update_minimap_for_doors(door, DoorType.POWER, scenario)
-    
+
     def power_to_door_type(self, door: Container, door_type: DoorType, scenario: str):
-        # set door and ensure door assets are present
+        # set door
         self.set_door_type(door, door_type, scenario)
-        
-        door_actor_folder = Path(door_type.door.actordefs[0]).parent.parent.as_posix()
-        for asset in self.editor.get_asset_names_in_folder(door_actor_folder):
-            self.editor.ensure_present_in_scenario(scenario, asset)
 
         # if needed, set shield and ensure shield assets are present
         if door_type.need_shield:
             life_comp = door.pComponents["LIFE"]
 
-            shieldL = self.create_shield(scenario, door, door_type.shield, "L")
-            shieldR = self.create_shield(scenario, door, door_type.shield, "R")
-            life_comp["wpLeftDoorShieldEntity"] = f"Root:pScenario:rEntitiesLayer:dctSublayers:default:dctActors:{shieldL.sName}"
-            life_comp["wpRightDoorShieldEntity"] = f"Root:pScenario:rEntitiesLayer:dctSublayers:default:dctActors:{shieldR.sName}"
+            shield_l = self.create_shield(scenario, door, door_type.shield, "L")
+            shield_r = self.create_shield(scenario, door, door_type.shield, "R")
+            life_comp[
+                "wpLeftDoorShieldEntity"] = f"Root:pScenario:rEntitiesLayer:dctSublayers:default:dctActors:{shield_l.sName}"
+            life_comp[
+                "wpRightDoorShieldEntity"] = f"Root:pScenario:rEntitiesLayer:dctSublayers:default:dctActors:{shield_r.sName}"
 
-            self.update_minimap_for_shield(shieldL, door_type.shield, "L", scenario)
-            self.update_minimap_for_shield(shieldR, door_type.shield, "R", scenario)
-
-            shield_actor_folder = Path(door_type.shield.actordefs[0]).parent.parent.as_posix()
-            for asset in self.editor.get_asset_names_in_folder(shield_actor_folder):
+            self.update_minimap_for_shield(shield_l, door_type.shield, "L", scenario)
+            self.update_minimap_for_shield(shield_r, door_type.shield, "R", scenario)
+        
+        # ensure assets are present
+        for folder in door_type.required_asset_folders:
+            for asset in self.editor.get_asset_names_in_folder(folder):
                 self.editor.ensure_present_in_scenario(scenario, asset)
-    
+
     def set_door_type(self, door: Container, door_type: DoorType, scenario: str):
-        # set actor def to two sided actordef
+        # set actor def to two-sided actordef
         door.oActorDefLink = f"actordef:{door_type.door.actordefs[0]}"
         self.update_minimap_for_doors(door, door_type, scenario)
 
@@ -283,14 +296,13 @@ class DoorPatcher:
         return shield
 
     def update_minimap_for_doors(self, door: Container, door_type: DoorType, scenario: str):
-        map = self.editor.get_scenario_map(scenario)
-        mapDoors = map.raw.Root.mapDoors
-        patchedContainer = door_type.door.minimapData.create_mapDoor(door.vPos)
-        mapDoors[door.sName] = patchedContainer
-    
+        scenario_map = self.editor.get_scenario_map(scenario)
+        map_doors = scenario_map.raw.Root.mapDoors
+        patched_container = door_type.door.minimapData.create_map_door(door.vPos)
+        map_doors[door.sName] = patched_container
+
     def update_minimap_for_shield(self, shield: Container, shield_type: ActorData, dir: str, scenario: str):
-        map = self.editor.get_scenario_map(scenario)
-        mapBlockages = map.raw.Root.mapBlockages
-        patchedContainer = shield_type.minimapData.create_mapBlockage(shield.vPos, dir)
-        mapBlockages[shield.sName] = patchedContainer
-    
\ No newline at end of file
+        scenario_map = self.editor.get_scenario_map(scenario)
+        map_blockages = scenario_map.raw.Root.mapBlockages
+        patched_container = shield_type.minimapData.create_map_blockage(shield.vPos, dir)
+        map_blockages[shield.sName] = patched_container
