@@ -53,6 +53,10 @@ end
 
 function s060_quarantine.InitFromBlackboard()
   Game.ReinitPlayerFromBlackboard()
+  if not Scenario.ReadFromBlackboard(Scenario.RandoTrueXRelease, false) then
+    Game.SetXparasite(false)
+    Blackboard.SetProp("GAME_PROGRESS", "QUARENTINE_OPENED", "b", false)
+  end
   s060_quarantine.CheckGatesOpened()
 end
 
@@ -99,6 +103,7 @@ function s060_quarantine.OnEntityGenerated(_ARG_0_, _ARG_1_)
 end
 
 function s060_quarantine.OnEnter_XParasite_Activated(_ARG_0_, _ARG_1_)
+  Scenario.WriteToBlackboard(Scenario.RandoTrueXRelease, "b", true)
   Game.SetXparasite(true)
 end
 
@@ -175,6 +180,8 @@ function s060_quarantine.OnSubAreaChange(old_subarea, old_actorgroup, new_subare
       end
     end
   end
+
+  Scenario.UpdateProgressiveItemModels()
 end
 
 
@@ -361,8 +368,13 @@ end
 
 function s060_quarantine.OnUsableCancelUse(actor)
   Scenario.ResetGlobalTeleport(actor)
+  Scenario.CheckWarpToStart(actor)
 end
 
 function s060_quarantine.OnUsableUse(actor)
+  if actor.sName == "wagontrain_forest_000" and Init.bDefaultXRelease then
+    Game.SetXparasite(true)
+    Blackboard.SetProp("GAME_PROGRESS", "QUARENTINE_OPENED", "b", true)
+  end
   Scenario.SetTeleportalUsed(actor)
 end
